-- Copyright © 2011 John Kirkpatrick                                   
--                                                                     
-- This file is part of the Create Annual Collections Plugin for
-- Adobe Lightroom 3.                                                 
--                                                                     
-- Create Annual Collections is free software: you can redistribute it and/or  
-- modify it under the terms of the GNU General Public License as      
-- published by the Free Software Foundation, either version 3 of the  
-- License, or (at your option) any later version.                     
--                                                                     
-- This software is distributed in the hope that it will be useful,     
-- but WITHOUT ANY WARRANTY; without even the implied warranty of      
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       
-- GNU General Public License for more details.                        
--                                                                     
-- You should have received a copy of the GNU General Public License   
-- in the file gpl.txt along with Create Annual Collections.
-- If not, please see <http://www.gnu.org/licenses/>.                                                          

local LrFunctionContext = import 'LrFunctionContext'
local LrBinding = import 'LrBinding'
local LrDialogs = import 'LrDialogs'
local LrView = import 'LrView'
local LrDate = import 'LrDate'
local LrLogger = import 'LrLogger'
local LrApplication = import 'LrApplication'
local LrTasks = import "LrTasks"

TileBarn = {}

local log = LrLogger("TB_createAnnualCollections")
log:enable("logfile")
log:trace("Started execution at " .. LrDate.timeToW3CDate(LrDate.currentTime()))

function TileBarn.createAnnualSmartCollections()
  LrTasks.startAsyncTask(function()
    local result = LrFunctionContext.callWithContext("showCustomDialog", function (context)
      local cat = LrApplication.activeCatalog()
      cat:withWriteAccessDo("Create annual collection set", function()
        -- Get a list of existing collection sets, and find the default parent set, if it exists
        local combo_items = {}
        local existingCollectionSets = cat:getChildCollectionSets()
        for _, s in ipairs(existingCollectionSets) do
	        combo_items[#combo_items+1] = s:getName()
	      end
	
	      -- Define the observable properties
	      local props = LrBinding.makePropertyTable(context)
	      props.setName = string.format("%d", LrDate.timeToUserFormat(LrDate.currentTime(), "%Y") + 1)
	      props.isLeapYear = (tonumber(props.setName) % 4) == 0 and (tonumber(props.setName) % 100) ~= 0
	      props.parentSetName = LOC "Smart Collections"
	
	      -- Define the UI
	      local f = LrView.osFactory()
	      local c = f:view
		              {
			              f:spacer {},
			              bind_to_object = props,
	                  spacing = f:control_spacing(),
	                  f:row
	                  {
		                  f:static_text {title = "Parent collection set:"},
		                  f:combo_box {items=combo_items, value=LrView.bind("parentSetName")},
			  			      },
	                  f:row
	                  {
		                  f:static_text {title = "Collection set:"},
		                  f:edit_field {value = LrView.bind("setName")},
						        },
						        f:row
						        {
							        f:checkbox {title = "Is leap year?", value = LrView.bind("isLeapYear")},
	                  }
	                }
	        -- Display the dialog
	        local result = LrDialogs.presentModalDialog({title="Create Annual Smart Collections", contents = c})
	
	        -- If the user has clicked OK, then create the collections
	        if result == "ok" then
		        log:trace("Creating collection set '" .. props.setName .. "' in '" .. props.parentSetName .. "'...")
		        if cat.hasWriteAccess then
				      log:trace("Write access has been obtained.")
				    else
				      log:error("Write access could not be obtained.")
				    end
				    -- Locate the parent collection set
				    local parentSet = nil
	          for _, s in ipairs(existingCollectionSets) do
		          if s:getName() == props.parentSetName then
						    log:trace("Parent collection set is '" .. s:getName() .. "'")
						    parentSet = s
				        break
			        end
		        end
		        -- Create the parent collection set
		        set = cat:createCollectionSet(props.setName, parentSet, false)
				    if set == nil then
				      LrDialogs.message("Tile Barn", "Unable to create collection set", "info")
				      log:error("Unable to create collection set")
				      return
				    end
				    log:trace("Collection set '" .. props.setName .. "' successfully created.")
				    -- Create month-by-month collections
				    for i=1, 12 do
				      local s = LrDate.timeToUserFormat(LrDate.timeFromComponents(props.setName, i, 1, 0, 0, 0, "local"), "%m-%b")
              -- Calculate last day of month
              local lastDay = 31
              if i == 2 then
	              if props.isLeapYear then lastDay = 29 else lastDay = 28 end
	            elseif i == 4 or i == 6 or i == 9 or i == 11 then lastDay = 30
	            end
				      local crit = {
						                 {
							                 criteria="captureTime",
						                   operation="in",
						                   value=LrDate.timeToUserFormat(LrDate.timeFromComponents(props.setName, i, 1, 0, 0, 0, "local"), "%Y-%m-%d"),
						                   value2=LrDate.timeToUserFormat(LrDate.timeFromComponents(props.setName, i, lastDay, 23, 59, 59, "local"), "%Y-%m-%d"),
						                 },
						                 combine="intersect",
						               }
				      local tmpCollection = cat:createSmartCollection(s, crit, set, false)
				      log:trace("  Smart collection " .. s .. " successfully created.")
				    end
			      LrDialogs.message("Tile Barn", "Annual smart collections successfully created.", "info")
	        else
		        LrDialogs.message("Info", "Cancelled", "info")
		        log:trace("Cancelled by user")
		      end
		      log:trace("Finished execution at " .. LrDate.timeToW3CDate(LrDate.currentTime()))
		    end)  -- withWriteAccessDo
		  end) -- startAsyncTask
	  end) -- callWithContext
end

TileBarn.createAnnualSmartCollections()
